@@ -1,25 +1,25 @@
 /*
+ * Copyright (C) 2004, 2005, 2007  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2003  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: loc_29.c,v 1.30.2.4 2003/10/09 07:32:42 marka Exp $ */
+/* $Id: loc_29.c,v 1.45 2007/06/19 23:47:17 tbox Exp $ */
 
 /* Reviewed: Wed Mar 15 18:13:09 PST 2000 by explorer */
 
-/* RFC 1876 */
+/* RFC1876 */
 
 #ifndef RDATA_GENERIC_LOC_29_C
 #define RDATA_GENERIC_LOC_29_C
@@ -55,8 +55,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	UNUSED(type);
 	UNUSED(rdclass);
 	UNUSED(origin);
-	UNUSED(downcase);
-	UNUSED(callbacks);
+	UNUSED(options);
 
 	/*
 	 * Defaults.
@@ -81,11 +80,11 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "N") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "N") == 0)
 		north = ISC_TRUE;
-	if (north || strcasecmp(token.value.as_pointer, "S") == 0)
+	if (north || strcasecmp(DNS_AS_STR(token), "S") == 0)
 		goto getlong;
-	m1 = strtol(token.value.as_pointer, &e, 10);
+	m1 = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0)
 		RETTOK(DNS_R_SYNTAX);
 	if (m1 < 0 || m1 > 59)
@@ -98,18 +97,19 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "N") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "N") == 0)
 		north = ISC_TRUE;
-	if (north || strcasecmp(token.value.as_pointer, "S") == 0)
+	if (north || strcasecmp(DNS_AS_STR(token), "S") == 0)
 		goto getlong;
-	s1 = strtol(token.value.as_pointer, &e, 10);
+	s1 = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.')
 		RETTOK(DNS_R_SYNTAX);
 	if (s1 < 0 || s1 > 59)
 		RETTOK(ISC_R_RANGE);
 	if (*e == '.') {
+		const char *l;
 		e++;
-		for (i = 0; i < 3 ; i++) {
+		for (i = 0; i < 3; i++) {
 			if (*e == 0)
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -117,10 +117,24 @@ fromtext_loc(ARGS_FROMTEXT) {
 			s1 *= 10;
 			s1 += tmp;
 		}
-		for ( ; i < 3 ; i++)
+		for (; i < 3; i++)
 			s1 *= 10;
-		if (*e != 0)
-			RETTOK(DNS_R_SYNTAX);
+		l = e;
+		while (*e != 0) {
+			if (decvalue(*e++) < 0)
+				RETTOK(DNS_R_SYNTAX);
+		}
+		if (*l != '\0' && callbacks != NULL) {
+			const char *file = isc_lex_getsourcename(lexer);
+			unsigned long line = isc_lex_getsourceline(lexer);
+
+			if (file == NULL)
+				file = "UNKNOWN";
+			(*callbacks->warn)(callbacks, "%s: %s:%u: '%s' extra "
+					   "precision digits ignored",
+					   "dns_rdata_fromtext", file, line,
+					   DNS_AS_STR(token));
+		}
 	} else
 		s1 *= 1000;
 	if (d1 == 90 && s1 != 0)
@@ -131,9 +145,9 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "N") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "N") == 0)
 		north = ISC_TRUE;
-	if (!north && strcasecmp(token.value.as_pointer, "S") != 0)
+	if (!north && strcasecmp(DNS_AS_STR(token), "S") != 0)
 		RETTOK(DNS_R_SYNTAX);
 
  getlong:
@@ -151,11 +165,11 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "E") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "E") == 0)
 		east = ISC_TRUE;
-	if (east || strcasecmp(token.value.as_pointer, "W") == 0)
+	if (east || strcasecmp(DNS_AS_STR(token), "W") == 0)
 		goto getalt;
-	m2 = strtol(token.value.as_pointer, &e, 10);
+	m2 = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0)
 		RETTOK(DNS_R_SYNTAX);
 	if (m2 < 0 || m2 > 59)
@@ -168,18 +182,19 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "E") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "E") == 0)
 		east = ISC_TRUE;
-	if (east || strcasecmp(token.value.as_pointer, "W") == 0)
+	if (east || strcasecmp(DNS_AS_STR(token), "W") == 0)
 		goto getalt;
-	s2 = strtol(token.value.as_pointer, &e, 10);
+	s2 = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.')
 		RETTOK(DNS_R_SYNTAX);
 	if (s2 < 0 || s2 > 59)
 		RETTOK(ISC_R_RANGE);
 	if (*e == '.') {
+		const char *l;
 		e++;
-		for (i = 0; i < 3 ; i++) {
+		for (i = 0; i < 3; i++) {
 			if (*e == 0)
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -187,10 +202,24 @@ fromtext_loc(ARGS_FROMTEXT) {
 			s2 *= 10;
 			s2 += tmp;
 		}
-		for ( ; i < 3 ; i++)
+		for (; i < 3; i++)
 			s2 *= 10;
-		if (*e != 0)
-			RETTOK(DNS_R_SYNTAX);
+		l = e;
+		while (*e != 0) {
+			if (decvalue(*e++) < 0)
+				RETTOK(DNS_R_SYNTAX);
+		}
+		if (*l != '\0' && callbacks != NULL) {
+			const char *file = isc_lex_getsourcename(lexer);
+			unsigned long line = isc_lex_getsourceline(lexer);
+
+			if (file == NULL)
+				file = "UNKNOWN";
+			(*callbacks->warn)(callbacks, "%s: %s:%u: '%s' extra "
+					   "precision digits ignored",
+					   "dns_rdata_fromtext",
+					   file, line, DNS_AS_STR(token));
+		}
 	} else
 		s2 *= 1000;
 	if (d2 == 180 && s2 != 0)
@@ -201,9 +230,9 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	if (strcasecmp(token.value.as_pointer, "E") == 0)
+	if (strcasecmp(DNS_AS_STR(token), "E") == 0)
 		east = ISC_TRUE;
-	if (!east && strcasecmp(token.value.as_pointer, "W") != 0)
+	if (!east && strcasecmp(DNS_AS_STR(token), "W") != 0)
 		RETTOK(DNS_R_SYNTAX);
 
  getalt:
@@ -212,7 +241,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	m = strtol(token.value.as_pointer, &e, 10);
+	m = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.' && *e != 'm')
 		RETTOK(DNS_R_SYNTAX);
 	if (m < -100000 || m > 42849672)
@@ -220,7 +249,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	cm = 0;
 	if (*e == '.') {
 		e++;
-		for (i = 0; i < 2 ; i++) {
+		for (i = 0; i < 2; i++) {
 			if (*e == 0 || *e == 'm')
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -231,7 +260,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 			else
 				cm += tmp;
 		}
-		for ( ; i < 2 ; i++)
+		for (; i < 2; i++)
 			cm *= 10;
 	}
 	if (*e == 'm')
@@ -259,7 +288,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 		isc_lex_ungettoken(lexer, &token);
 		goto encode;
 	}
-	m = strtol(token.value.as_pointer, &e, 10);
+	m = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.' && *e != 'm')
 		RETTOK(DNS_R_SYNTAX);
 	if (m < 0 || m > 90000000)
@@ -267,7 +296,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	cm = 0;
 	if (*e == '.') {
 		e++;
-		for (i = 0; i < 2 ; i++) {
+		for (i = 0; i < 2; i++) {
 			if (*e == 0 || *e == 'm')
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -275,7 +304,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 			cm *= 10;
 			cm += tmp;
 		}
-		for ( ; i < 2 ; i++)
+		for (; i < 2; i++)
 			cm *= 10;
 	}
 	if (*e == 'm')
@@ -286,7 +315,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 * We don't just multiply out as we will overflow.
 	 */
 	if (m > 0) {
-		for (exp = 0 ; exp < 7 ; exp++)
+		for (exp = 0; exp < 7; exp++)
 			if (m < poweroften[exp+1])
 				break;
 		man = m / poweroften[exp];
@@ -312,7 +341,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 		isc_lex_ungettoken(lexer, &token);
 		goto encode;
 	}
-	m = strtol(token.value.as_pointer, &e, 10);
+	m = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.' && *e != 'm')
 		RETTOK(DNS_R_SYNTAX);
 	if (m < 0 || m > 90000000)
@@ -320,7 +349,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	cm = 0;
 	if (*e == '.') {
 		e++;
-		for (i = 0; i < 2 ; i++) {
+		for (i = 0; i < 2; i++) {
 			if (*e == 0 || *e == 'm')
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -328,7 +357,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 			cm *= 10;
 			cm += tmp;
 		}
-		for ( ; i < 2 ; i++)
+		for (; i < 2; i++)
 			cm *= 10;
 	}
 	if (*e == 'm')
@@ -339,7 +368,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 * We don't just multiply out as we will overflow.
 	 */
 	if (m > 0) {
-		for (exp = 0 ; exp < 7 ; exp++)
+		for (exp = 0; exp < 7; exp++)
 			if (m < poweroften[exp+1])
 				break;
 		man = m / poweroften[exp];
@@ -363,7 +392,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 		isc_lex_ungettoken(lexer, &token);
 		goto encode;
 	}
-	m = strtol(token.value.as_pointer, &e, 10);
+	m = strtol(DNS_AS_STR(token), &e, 10);
 	if (*e != 0 && *e != '.' && *e != 'm')
 		RETTOK(DNS_R_SYNTAX);
 	if (m < 0 || m > 90000000)
@@ -371,7 +400,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	cm = 0;
 	if (*e == '.') {
 		e++;
-		for (i = 0; i < 2 ; i++) {
+		for (i = 0; i < 2; i++) {
 			if (*e == 0 || *e == 'm')
 				break;
 			if ((tmp = decvalue(*e++)) < 0)
@@ -379,7 +408,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 			cm *= 10;
 			cm += tmp;
 		}
-		for ( ; i < 2 ; i++)
+		for (; i < 2; i++)
 			cm *= 10;
 	}
 	if (*e == 'm')
@@ -390,7 +419,7 @@ fromtext_loc(ARGS_FROMTEXT) {
 	 * We don't just multiply out as we will overflow.
 	 */
 	if (m > 0) {
-		for (exp = 0 ; exp < 7 ; exp++)
+		for (exp = 0; exp < 7; exp++)
 			if (m < poweroften[exp+1])
 				break;
 		man = m / poweroften[exp];
@@ -534,7 +563,7 @@ fromwire_loc(ARGS_FROMWIRE) {
 	UNUSED(type);
 	UNUSED(rdclass);
 	UNUSED(dctx);
-	UNUSED(downcase);
+	UNUSED(options);
 
 	isc_buffer_activeregion(source, &sr);
 	if (sr.length < 1)
@@ -619,7 +648,7 @@ compare_loc(ARGS_COMPARE) {
 
 	dns_rdata_toregion(rdata1, &r1);
 	dns_rdata_toregion(rdata2, &r2);
-	return (compare_region(&r1, &r2));
+	return (isc_region_compare(&r1, &r2));
 }
 
 static inline isc_result_t
@@ -737,4 +766,29 @@ digest_loc(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
+static inline isc_boolean_t
+checkowner_loc(ARGS_CHECKOWNER) {
+
+	REQUIRE(type == 29);
+
+	UNUSED(name);
+	UNUSED(type);
+	UNUSED(rdclass);
+	UNUSED(wildcard);
+
+	return (ISC_TRUE);
+}
+
+static inline isc_boolean_t
+checknames_loc(ARGS_CHECKNAMES) {
+
+	REQUIRE(rdata->type == 29);
+
+	UNUSED(rdata);
+	UNUSED(owner);
+	UNUSED(bad);
+
+	return (ISC_TRUE);
+}
+
 #endif	/* RDATA_GENERIC_LOC_29_C */
